import { z } from "zod";
import { useForm } from "react-hook-form";
import { zodResolver } from "@hookform/resolvers/zod";
import { Link } from "@tanstack/react-router";
import { toast } from "@/hooks/useToast";
import { Button } from "@/components/ui/button";
import {
	Form,
	FormControl,
	FormDescription,
	FormField,
	FormItem,
	FormLabel,
	FormMessage
} from "@/components/ui/form";
import { Input } from "@/components/ui/input";
import { PhoneInput } from "@/components/ui/phone-number-input";
import useAuth from "@/hooks/useAuth";

const profileFormSchema = z.object({
	username: z
		.string({
			required_error: "Please enter your desired username"
		})
		.min(3, {
			message: "Username must be at least 3 characters."
		})
		.max(64, {
			message: "Username must not be longer than 64 characters."
		}),
	email: z
		.string()
		.email({
			message: "Please enter your email address."
		})
		.email(),
	phone_number: z
		.string({
			required_error: "Please input a valid phone number."
		})
		.min(2, {
			message: "Phone number must be at least 2 characters."
		})
		.max(16, {
			message: "Phone number must not be longer than 16 characters."
		}),
	first_name: z
		.string()
		.max(64, {
			message: "First name must not be longer than 64 characters."
		})
		.optional(),
	last_name: z
		.string()
		.max(64, {
			message: "Last name must not be longer than 64 characters."
		})
		.optional()
});

type ProfileFormValues = z.infer<typeof profileFormSchema>;

export default function ProfileForm() {
	const { user, updateAccountMutation } = useAuth();

	const defaultValues: ProfileFormValues = {
		username: user?.username ?? "",
		email: user?.email ?? "",
		phone_number: user?.phone_number ?? "",
		first_name: user?.first_name ?? "",
		last_name: user?.last_name ?? ""
	};

	const form = useForm<ProfileFormValues>({
		resolver: zodResolver(profileFormSchema),
		defaultValues,
		mode: "onChange"
	});

	function onSubmit(data: ProfileFormValues) {
		updateAccountMutation.mutate(data);
		toast({
			title: "You submitted the following values:",
			description: (
				<pre className="mt-2 w-[340px] rounded-md bg-slate-950 p-4">
					<code className="text-white">{JSON.stringify(data, null, 2)}</code>
				</pre>
			)
		});
	}

	return (
		<Form {...form}>
			<form onSubmit={form.handleSubmit(onSubmit)} className="space-y-8">
				<FormField
					control={form.control}
					name="username"
					render={({ field }) => (
						<FormItem>
							<FormLabel>Username</FormLabel>
							<FormControl>
								<Input placeholder="Username" {...field} />
							</FormControl>
							<FormDescription>
								This is your public display name. It can be your real name or a
								pseudonym.
							</FormDescription>
							<FormMessage />
						</FormItem>
					)}
				/>
				<FormField
					control={form.control}
					name="email"
					render={({ field }) => (
						<FormItem>
							<FormLabel>Email</FormLabel>
							<FormControl>
								<Input type="email" {...field} />
							</FormControl>
							<FormDescription>
								You can manage verified email addresses in your{" "}
								<Link to="/">email settings</Link>.
							</FormDescription>
							<FormMessage />
						</FormItem>
					)}
				/>
				<FormField
					control={form.control}
					name="first_name"
					render={({ field }) => (
						<FormItem>
							<FormLabel>First Name</FormLabel>
							<FormControl>
								<Input placeholder="John" {...field} />
							</FormControl>
							<FormMessage />
						</FormItem>
					)}
				/>
				<FormField
					control={form.control}
					name="last_name"
					render={({ field }) => (
						<FormItem>
							<FormLabel>Last Name</FormLabel>
							<FormControl>
								<Input placeholder="Doe" {...field} />
							</FormControl>
							<FormMessage />
						</FormItem>
					)}
				/>

				<FormField
					control={form.control}
					name="phone_number"
					render={({ field }) => (
						<FormItem>
							<FormLabel>Phone number</FormLabel>
							<FormControl>
								<PhoneInput {...field} />
							</FormControl>
							<FormMessage />
						</FormItem>
					)}
				/>
				<Button type="submit">Update profile</Button>
			</form>
		</Form>
	);
}
